@@ -38,6 +38,113 @@
 #define DIRCHR '/'
 #endif
 
+gchar *get_filename_on_disk_encoding(const gchar *utf8filename) {
+	if (utf8filename) {
+		GError *gerror=NULL;
+		gsize b_written;
+		gchar *ondiskencoding = g_filename_from_utf8(utf8filename,-1, NULL,&b_written,&gerror);
+		if (gerror) {
+			g_print(_("Bluefish has trouble reading the filenames. Try to set the environment variable G_BROKEN_FILENAMES=1\n"));
+			ondiskencoding = g_strdup(utf8filename);
+		}
+		return ondiskencoding;
+	}
+	return NULL;
+}
+
+gchar *get_utf8filename_from_on_disk_encoding(const gchar *encodedname) {
+	if (encodedname) {
+		GError *gerror=NULL;
+		gsize b_written;
+		gchar *ondiskencoding = g_filename_to_utf8(encodedname,-1, NULL,&b_written,&gerror);
+		if (gerror) {
+			g_print(_("Bluefish has trouble reading the filenames. Try to set the environment variable G_BROKEN_FILENAMES=1\n"));
+			ondiskencoding = g_strdup(encodedname);
+		}
+		return ondiskencoding;
+	}
+	return NULL;
+}
+
+gboolean string_is_color(const gchar *color) {
+	GdkColor gcolor;
+	return gdk_color_parse(color, &gcolor);
+}
+
+static void fill_rwx(short unsigned int bits, char *chars) {
+	chars[0] = (bits & S_IRUSR) ? 'r' : '-';
+	chars[1] = (bits & S_IWUSR) ? 'w' : '-';
+	chars[2] = (bits & S_IXUSR) ? 'x' : '-';
+}
+static void fill_setid(short unsigned int bits, char *chars) {
+#ifdef S_ISUID
+	if (bits & S_ISUID) {
+		/* Set-uid, but not executable by owner.  */
+		if (chars[3] != 'x') chars[3] = 'S';
+		else chars[3] = 's';
+	}
+#endif
+#ifdef S_ISGID
+	if (bits & S_ISGID) {
+		/* Set-gid, but not executable by group.  */
+		if (chars[6] != 'x') chars[6] = 'S';
+		else chars[6] = 's';
+	}
+#endif
+#ifdef S_ISVTX
+	if (bits & S_ISVTX) {
+		/* Sticky, but not executable by others.  */
+		if (chars[9] != 'x') chars[9] = 'T';
+		else chars[9] = 't';
+	}
+#endif
+}
+gchar *filemode_to_string(mode_t statmode) {
+	gchar *str = g_malloc0(10);
+ 	/* following code "adapted" from GNU filemode.c program */
+	fill_rwx((statmode & 0700) << 0, &str[0]);
+	fill_rwx((statmode & 0070) << 3, &str[3]);
+	fill_rwx((statmode & 0007) << 6, &str[6]);
+	fill_setid(statmode, str);
+	return str;
+}
+
+
+/**
+ * return_root_with_protocol:
+ * @url: #const gchar* with the url 
+ *
+ * returns the root of the url, including its trailing slash
+ * this might be in the form
+ * - "protocol://server:port/"
+ * - "/"
+ * - NULL, if the url contains no url, nor does it start with a / character
+ *
+ * if there is no trailing slash, this function will return the root WITH a
+ * trailing slash appended!!
+ *
+ * Return value: #gchar* newly allocated, or NULL
+ */
+gchar *return_root_with_protocol(const gchar *url) {
+	gchar *q;
+	if (!url) return NULL;
+	q = strchr(url,':');
+	if (q && *(q+1)=='/' && *(q+2)=='/' && *(q+3)!='\0') {
+		/* we have a protocol */
+		gchar *root = strchr(q+3, '/');
+		if (root) return g_strndup(url, root - url + 1);
+		/* if there is no third slash character, we probably
+		have an url like http://someserver so we will append 
+		the slash ourselves */
+		return g_strconcat(url, "/",NULL);
+	} else if (url[0] == '/') {
+		/* no protocol, return / */
+		return g_strdup("/");
+	}
+	/* no root known */
+	return NULL;
+}
+
 /**
  * pointer_switch_addresses:
  * a: #gpointer;
@@ -83,9 +190,46 @@ void list_switch_order(GList *first, GList *second) {
  * @dest: a #gchar * containing the destination filename
  * 
  * copies the contents of the file source to dest
+ * this function is Gnome-VFS aware, so it will work on URI's
  * 
  * Return value: gboolean, TRUE if the function succeeds
  **/
+#ifdef HAVE_GNOME_VFS
+#define BYTES_TO_PROCESS 8196
+gboolean file_copy(gchar *source, gchar *dest) {
+	GnomeVFSHandle *read_handle, *write_handle;
+	GnomeVFSFileSize bytes_read, bytes_written;
+	guint buffer[BYTES_TO_PROCESS];
+	GnomeVFSResult result;
+	gchar *OnDiEn_source, *OnDiEn_dest;
+	OnDiEn_source = get_filename_on_disk_encoding(source);
+	OnDiEn_dest = get_filename_on_disk_encoding(dest);
+	
+	result = gnome_vfs_open(&read_handle, OnDiEn_source, GNOME_VFS_OPEN_READ);
+	g_free(OnDiEn_source);
+	if (result != GNOME_VFS_OK) return FALSE;
+	result = gnome_vfs_create(&write_handle, OnDiEn_dest, GNOME_VFS_OPEN_WRITE, FALSE, 0644);
+	g_free(OnDiEn_dest);
+	if (result != GNOME_VFS_OK) {
+		gnome_vfs_close(read_handle);
+		return FALSE;
+	}
+	result = gnome_vfs_read (read_handle, buffer, BYTES_TO_PROCESS, &bytes_read);
+	while (result == GNOME_VFS_OK) {
+		result = gnome_vfs_write (write_handle, buffer, bytes_read, &bytes_written);
+		if (result != GNOME_VFS_OK || bytes_written != bytes_read) {
+			DEBUG_MSG("file_copy, return FALSE, write result=%d, written=%ld, read=%ld\n",result,(long)bytes_written,(long)bytes_read);
+			gnome_vfs_close(write_handle);
+			gnome_vfs_close(read_handle);
+			return FALSE;
+		}
+		result = gnome_vfs_read(read_handle, buffer, BYTES_TO_PROCESS, &bytes_read);
+	}
+	gnome_vfs_close(write_handle);
+	gnome_vfs_close(read_handle);
+	return TRUE;
+}
+#else  /* HAVE_GNOME_VFS */
 gboolean file_copy(gchar *source, gchar *dest) {
 #ifdef DEVELOPMENT
 	g_assert(source);
@@ -93,12 +237,17 @@ gboolean file_copy(gchar *source, gchar *dest) {
 #endif
 	int c;
 	FILE *in, *out;
-	
-	in = fopen(source, "r");
+	gchar *OnDiEn_source, *OnDiEn_dest;
+	OnDiEn_source = get_filename_on_disk_encoding(source);
+	OnDiEn_dest = get_filename_on_disk_encoding(dest);
+
+	in = fopen(OnDiEn_source, "r");
+	g_free(OnDiEn_source);
 	if (!in) {
 		return FALSE;
 	}
-	out = fopen(dest, "w");
+	out = fopen(OnDiEn_dest, "w");
+	g_free(OnDiEn_dest);
 	if (!out) {
 		fclose(in);
 		return FALSE;
@@ -110,6 +259,46 @@ gboolean file_copy(gchar *source, gchar *dest) {
 	fclose(out);
 	return TRUE;
 }
+#endif /* HAVE_GNOME_VFS */
+
+static gint length_common_prefix(gchar *first, gchar *second) {
+	gint i=0;
+	while (first[i] == second[i] && first[i] != '\0') {
+		i++;
+	}
+	return i;
+}
+/**
+ * find_common_prefix_in_stringlist:
+ * @stringlist: a #GList* with strings
+ * 
+ * tests every string in stringlist, and returns the length of the 
+ * common prefix all these strings have
+ *
+ * This is for example useful to find out if a list of filenames
+ * share the same base directory
+ * 
+ * Return value: #gint with number of common characters
+ **/
+gint find_common_prefixlen_in_stringlist(GList *stringlist) {
+	gchar *firststring;
+	gint commonlen;
+	GList *tmplist;
+	tmplist = g_list_first(stringlist);
+	firststring = (gchar *)tmplist->data;
+	commonlen = strlen(firststring);
+	tmplist = g_list_next(tmplist);
+	while(tmplist){
+		gint testlen;
+		gchar *secondstring = (gchar *)tmplist->data;
+		testlen = length_common_prefix(firststring, secondstring);
+		if (testlen < commonlen) {
+			commonlen = testlen;
+		}
+		tmplist = g_list_next(tmplist);
+	}
+	return commonlen;
+}
 /**
  * append_string_to_file:
  * @filename: a #gchar * containing the destination filename
@@ -117,13 +306,16 @@ gboolean file_copy(gchar *source, gchar *dest) {
  * 
  * opens the file filename in append mode, and appends the string
  * no newline or anything else is appended, just the string
+ *
+ * DOES NOT YET SUPPORT GNOME_VFS !!!
  * 
  * Return value: gboolean, TRUE if the function succeeds
  **/
 gboolean append_string_to_file(gchar *filename, gchar *string) {
 	FILE *out;
-
-	out = fopen(filename, "a");
+	gchar *ondiskencoding = get_filename_on_disk_encoding(filename);
+	out = fopen(ondiskencoding, "a");
+	g_free(ondiskencoding);
 	if (!out) {
 		DEBUG_MSG("append_to_file, could not open file %s for append\n", filename);
 		return FALSE;
@@ -132,26 +324,71 @@ gboolean append_string_to_file(gchar *filename, gchar *string) {
 	fclose(out);
 	return TRUE;
 }
+/**
+ * countchars:
+ * @string: a gchar * to count the chars in
+ * @chars: a gchar * with the characters you are interested in
+ *
+ * this function will count every character in string that is also in chars
+ * 
+ * Return value: guint with the number of characters found
+ **/
+guint countchars(const gchar *string, const gchar *chars) {
+	guint count=0;
+	gchar *newstr = strpbrk(string, chars);
+	while(newstr) {
+		count++;
+		newstr = strpbrk(++newstr, chars);
+	}
+	DEBUG_MSG("countchars, returning %d\n",count);
+	return count;
+}
+static gint table_convert_char2int_backend(Tconvert_table *table, const gchar *my_char
+		, Ttcc2i_mode mode, int (*func)(const gchar *arg1, const gchar *arg2) ) {
+	Tconvert_table *entry;
+	entry = table;
+	while (entry->my_char) {
+		if (func(my_char,entry->my_char)==0) {
+			return entry->my_int;
+		}
+		entry++;
+	}
+	return -1;
+}
+static int strfirstchar(const gchar *mychar, const gchar *tablechar) {
+	return mychar[0] - tablechar[0];
+}
+static int strmycharlen(const gchar *mychar, const gchar *tablechar) {
+	return strncmp(mychar,tablechar,strlen(mychar));
+}
+static int strfull_match_gettext(const gchar *mychar, const gchar *tablechar) {
+	return strcmp(mychar,_(tablechar));
+}
 /**
  * table_convert_char2int:
  * @table: a #tconvert_table * with strings and integers
  * @my_char: a #gchar * containing the string to convert
+ * @mode: #Ttcc2i_mode
  * 
  * this function can be used to translate a string from some set (in table)
  * to an integer
  * 
  * Return value: gint, found in table, or -1 if not found
  **/
-gint table_convert_char2int(Tconvert_table *table, gchar *my_char) {
-	Tconvert_table *entry;
-	entry = table;
-	while (entry->my_char) {
-		if (strcmp(entry->my_char, my_char) == 0) {
-			return entry->my_int;
-		}
-		entry++;
+gint table_convert_char2int(Tconvert_table *table, const gchar *my_char, Ttcc2i_mode mode) {
+	switch (mode) {
+	case tcc2i_firstchar:
+		return table_convert_char2int_backend(table,my_char,mode,strfirstchar);
+	case tcc2i_mycharlen:
+		return table_convert_char2int_backend(table,my_char,mode,strmycharlen);
+	case tcc2i_full_match:
+		return table_convert_char2int_backend(table,my_char,mode,strcmp);
+	case tcc2i_full_match_gettext:
+		return table_convert_char2int_backend(table,my_char,mode,strfull_match_gettext);
+	default:
+		DEBUG_MSG("bug in call to table_convert_char2int()\n");
+		return -1;
 	}
-	return -1;
 }
 /**
  * table_convert_int2char:
@@ -177,50 +414,172 @@ gchar *table_convert_int2char(Tconvert_table *table, gint my_int) {
 	return NULL;
 }
 /**
- * replace_string_printflike:
+ * expand_string:
  * @string: a formatstring #gchar * to convert
- * @table: a #Tconvert_table * to use for conversion
+ * @specialchar: a const char to use as 'delimited' or 'special character'
+ * @table: a #Tconvert_table * array to use for conversion
  * 
- * this function can convert a format string with %0, %1 etc. 
- * into the final string, where each %number entry is replaced 
+ * this function can convert a format string with %0, %1, or \n, \t 
+ * into the final string, where each %number or \char entry is replaced 
  * with the string found in table
+ *
+ * so this function is the backend for unescape_string() and 
+ * for replace_string_printflike()
+ *
+ * table is an array with last entry {0, NULL}
  * 
  * Return value: a newly allocated gchar * with the resulting string
  **/
-gchar *replace_string_printflike(const gchar *string, Tconvert_table *table) {
+gchar *expand_string(const gchar *string, const char specialchar, Tconvert_table *table) {
 	gchar *p, *prev, *stringdup;
 	gchar *tmp, *dest = g_strdup("");
 
 	stringdup = g_strdup(string); /* we make a copy so we can set some \0 chars in the string */
 	prev = stringdup;
-	DEBUG_MSG("replace_string_printflike, string='%s'\n", string);
-	p = strchr(prev, '%');
+	DEBUG_MSG("expand_string, string='%s'\n", string);
+	p = strchr(prev, specialchar);
 	while (p) {
 		gchar *converted;
 		tmp = dest;
 		*p = '\0'; /* set a \0 at this point, the pointer prev now contains everything up to the current % */
-		DEBUG_MSG("replace_string_printflike, prev='%s'\n", prev);
+		DEBUG_MSG("expand_string, prev='%s'\n", prev);
 		p++;
-		if (*p == '%') {
-			converted = "%";
-		} else {
-			converted = table_convert_int2char(table, *p);
-		}
-		DEBUG_MSG("replace_string_printflike, converted='%s'\n", converted);
+		converted = table_convert_int2char(table, *p);
+		DEBUG_MSG("expand_string, converted='%s'\n", converted);
 		dest = g_strconcat(dest, prev, converted, NULL);
 		g_free(tmp);
 		prev = ++p;
-		p = strchr(p, '%');
+		p = strchr(p, specialchar);
 	}
 	tmp = dest;
 	dest = g_strconcat(dest, prev, NULL); /* append the end to the current string */
 	g_free(tmp);
 	
 	g_free(stringdup);
-	DEBUG_MSG("replace_string_printflike, dest='%s'\n", dest);
+	DEBUG_MSG("expand_string, dest='%s'\n", dest);
 	return dest;
 }
+gchar *replace_string_printflike(const gchar *string, Tconvert_table *table) {
+	return expand_string(string,'%',table);
+}
 
+static gint tablesize(Tconvert_table *table) {
+	Tconvert_table *tmpentry = table;
+	while (tmpentry->my_char) tmpentry++;
+	return (tmpentry - table);
+}
+/* for now this function can only unexpand strings with tables that contain only
+single character strings like "\n", "\t" etc. */
+gchar *unexpand_string(const gchar *original, const char specialchar, Tconvert_table *table) {
+	gchar *tmp, *tosearchfor, *retval, *prev, *dest, *orig;
+	Tconvert_table *tmpentry;
+	
+	orig = g_strdup(original);
+	DEBUG_MSG("original='%s', strlen()=%d\n",original,strlen(original));
+	tosearchfor = g_malloc(tablesize(table)+1);
+	DEBUG_MSG("tablesize(table)=%d, alloc'ed %d bytes for tosearchfor\n",tablesize(table), tablesize(table)+1);
+	tmp = tosearchfor;
+	tmpentry = table;
+	while(tmpentry->my_char != NULL) {
+		*tmp = tmpentry->my_char[0]; /* we fill the search string with the first character */
+		tmpentry++;
+		tmp++;
+	}
+	*tmp = '\0';
+	DEBUG_MSG("unexpand_string, tosearchfor='%s'\n",tosearchfor);
+	DEBUG_MSG("alloc'ing %d bytes\n", (countchars(original, tosearchfor) + strlen(original) + 1));
+	retval = g_malloc((countchars(original, tosearchfor) + strlen(original) + 1) * sizeof(gchar));
+	dest = retval;
+	prev = orig;
+	/* now we go trough the original till we hit specialchar */
+	tmp = strpbrk(prev, tosearchfor);
+	while (tmp) {
+		gint len = tmp - prev;
+		gint mychar = table_convert_char2int(table, tmp, tcc2i_firstchar);
+		DEBUG_MSG("unexpand_string, tmp='%s', prev='%s'\n",tmp, prev);
+		if (mychar == -1) mychar = *tmp;
+		DEBUG_MSG("unexpand_string, copy %d bytes and advancing dest\n",len);
+		memcpy(dest, prev, len);
+		dest += len;
+		*dest = specialchar;
+		dest++;
+		*dest = mychar;
+		dest++;
+		prev=tmp+1;
+		DEBUG_MSG("prev now is '%s'\n",prev);
+		tmp = strpbrk(prev, tosearchfor);
+	}
+	DEBUG_MSG("unexpand_string, copy the rest (%s) to dest\n",prev);
+	memcpy(dest,prev,strlen(prev)+1); /* this will also make sure there is a \0 at the end */
+	DEBUG_MSG("unexpand_string, retval='%s'\n",retval);
+	g_free(orig);
+	g_free(tosearchfor);
+	return retval;
+}
+/* if you change this table, please change escape_string() and unescape_string() and new_convert_table() in the same way */
+static Tconvert_table standardescapetable [] = {
+	{'n', "\n"},
+	{'t', "\t"},
+	{'\\', "\\"},
+	{'f', "\f"},
+	{'r', "\r"},
+	{'a', "\a"},
+	{'b', "\b"},
+	{'v', "\v"},
+	{'n', "\n"},
+	{':', ":"}, /* this double entry is there to make unescape_string and escape_string work efficient */
+	{0, NULL}
+};
+gchar *unescape_string(const gchar *original, gboolean escape_colon) {
+	gchar *string, *tmp=NULL;
+	if (!escape_colon) {
+		tmp = standardescapetable[9].my_char;
+		standardescapetable[9].my_char = NULL;
+	}
+	string = expand_string(original,'\\',standardescapetable);
+	if (!escape_colon) {
+		standardescapetable[9].my_char = tmp;
+	}
+	return string;
+}
+gchar *escape_string(const gchar *original, gboolean escape_colon) {
+	gchar *string, *tmp=NULL;
+	if (!escape_colon) {
+		tmp = standardescapetable[9].my_char;
+		standardescapetable[9].my_char = NULL;
+	}
+	string = unexpand_string(original,'\\',standardescapetable);
+	if (!escape_colon) {
+		standardescapetable[9].my_char = tmp;
+	}
+	return string;
+}
+Tconvert_table *new_convert_table(gint size, gboolean fill_standardescape) {
+	gint realsize = (fill_standardescape) ? size + 10 : size +1;
+	Tconvert_table * tct = g_new(Tconvert_table, realsize);
+	DEBUG_MSG("new_convert_table, size=%d, realsize=%d\n",size,realsize);
+	if (fill_standardescape) {
+		gint i;
+		for (i=size;i<realsize;i++) {
+			tct[i].my_int = standardescapetable[i-size].my_int;
+			tct[i].my_char = g_strdup(standardescapetable[i-size].my_char);
+		}
+		tct[i].my_char = NULL;
+	} else {
+		tct[size].my_char = NULL;
+	}
+	return tct;
+}
+void free_convert_table(Tconvert_table *tct) {
+	Tconvert_table *tmp = tct;
+	while (tmp->my_char) {
+		DEBUG_MSG("free_convert_table, my_char=%s\n",tmp->my_char);
+		g_free(tmp->my_char);
+		tmp++;
+	}
+	DEBUG_MSG("free_convert_table, free table %p\n",tct);
+	g_free(tct);
+}
 /**************************************************/
 /* byte offset to UTF8 character offset functions */
 /**************************************************/
@@ -300,25 +659,6 @@ guint utf8_byteoffset_to_charsoffset_cached(gchar *string, glong byteoffset) {
 	return retval;
 }
 
-/**
- * countchars:
- * @string: a gchar * to count the chars in
- * @chars: a gchar * with the characters you are interested in
- *
- * this function will count every character in string that is also in chars
- * 
- * Return value: guint with the number of characters found
- **/
-guint countchars(gchar *string, gchar *chars) {
-	guint count=0;
-	gchar *newstr = strpbrk(string, chars);
-	while(newstr) {
-		count++;
-		newstr = strpbrk(++newstr, chars);
-	}
-	return count;
-}
-
 /**
  * escapestring:
  * @original: a gchar * to escape
@@ -329,12 +669,12 @@ guint countchars(gchar *string, gchar *chars) {
  * 
  * Return value: a newly allocated gchar * that is escaped
  **/
-gchar *escapestring(gchar *original, gchar delimiter)
+/*gchar *old_escapestring(gchar *original, gchar delimiter)
 {
 	gchar *tmp, *newstring, *escapedchars;
 	guint newsize, pos=0;
 
-	/* count the size of the new string */
+	* count the size of the new string *
 	escapedchars = g_strdup_printf("\n\t\\%c", delimiter);
 	DEBUG_MSG("escapestring, escapedchars=%s, extra length=%d\n", escapedchars, countchars(original, escapedchars));
 	newsize = countchars(original, escapedchars) + strlen(original) + 1;
@@ -373,10 +713,10 @@ gchar *escapestring(gchar *original, gchar delimiter)
 	}
 	DEBUG_MSG("escapestring, newstring = %s\n", newstring);
 	return newstring;
-}
+}*/
 
 /**
- * escapestring:
+ * unescapestring:
  * @original: a gchar * to unescape
  *
  * this function will backslash unescape \n, \t, and \\ sequences to 
@@ -385,16 +725,12 @@ gchar *escapestring(gchar *original, gchar delimiter)
  * 
  * Return value: a newly allocated gchar * that is unescaped
  **/
-gchar *unescapestring(gchar *original)
+/*gchar *old_unescapestring(gchar *original)
 {
 	gchar *tmp1, *tmp2, *newstring;
 	guint newsize;
 	gint escaped;
 
-	/* note: to split string into an array,
-	 * use string_to_array().
-	 */
-
 	newsize = strlen(original) + 1;
 	newstring = g_malloc0(newsize * sizeof(gchar));
 	DEBUG_MSG("unescapestring, original=%s, newsize = %d\n", original, newsize);
@@ -430,7 +766,7 @@ gchar *unescapestring(gchar *original)
 	}
 	DEBUG_MSG("unescapestring, newstring = %s\n", newstring);
 	return newstring;
-}
+}*/
 
 /**
  * change_dir:
@@ -683,6 +1019,9 @@ gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepat
  * else it will use basedir if available, else the current dir
  * to add to the filename to form the full path
  *
+ * for URL's it will simply return a strdup(), except for file:// URL's, 
+ * there the file:// bit is stripped
+ *
  * it does use most_efficient_filename() to remote unwanted dir/../ entries
  *
  * Return value: a newly allocated gchar * with the full path
@@ -692,6 +1031,15 @@ gchar *create_full_path(gchar * filename, gchar *basedir) {
 	gchar *tmpcdir;
 
 	DEBUG_MSG("create_full_path, filename=%s, basedir=%s\n", filename, basedir);
+#ifdef HAVE_GNOME_VFS
+	if (strchr(filename, ':') != NULL) {
+		DEBUG_MSG("create_full_path, %s is an URI\n",filename);
+		if (strlen(filename)>7 && strncmp(filename, "file://", 7)==0) {
+			return g_strdup(filename+7);
+		}
+		return g_strdup(filename);
+	}
+#endif /* HAVE_GNOME_VFS */
 	if (g_path_is_absolute(filename)) {
 		absolute_filename = g_strdup(filename);
 	} else {
@@ -752,37 +1100,46 @@ gchar *path_get_dirname_with_ending_slash(const gchar *filename) {
  * file_exists_and_readable:
  * @filename: a #const gchar * with a file path
  *
- * tests if the file exists, using stat(), and is readable, the last
+ * tests if the file exists, and  if it is readable, the last
  * check is not reliable, it does not check all the groups you are
  * in, so change this function before you rely on that check!
  *
+ * this function is Gnome-VFS aware, so it will work on URI's
+ *
  * Return value: gboolean, TRUE if readable, else FALSE
  **/
-gboolean file_exists_and_readable(const gchar * filename)
-{
-	struct stat naamstat;
-
+gboolean file_exists_and_readable(const gchar * filename) {
+	gchar *ondiskencoding;
+	gboolean retval=TRUE;
 #ifdef DEVELOPMENT
 	g_assert(filename);
 #endif
-
 	if (!filename || strlen(filename) < 2) {
 		DEBUG_MSG("file_exists_and_readable, strlen(filename) < 2 or no filename!!!!\n");
 		return FALSE;
 	}
 	DEBUG_MSG("file_exists_and_readable, filename(%p)=\"%s\", strlen(filename)=%d\n", filename, filename, strlen(filename));
+	ondiskencoding = get_filename_on_disk_encoding(filename);
+	DEBUG_MSG("file_exists_and_readable, ondiskencoding='%s'\n",ondiskencoding);
 #ifndef WIN32
-	if ((stat(filename, &naamstat) == -1) && (errno == ENOENT)) {
-/*              if(naamstat.st_mode & S_IREAD) { */
-		return FALSE;
-/*              }
-   return(0); */
-	} else {
-		return TRUE;
+#ifdef HAVE_GNOME_VFS
+	{
+		GnomeVFSURI* uri = gnome_vfs_uri_new(ondiskencoding);
+		retval = gnome_vfs_uri_exists(uri);
+		gnome_vfs_uri_unref(uri);
+		DEBUG_MSG("file_exists_and_readable, return %d for %s\n",retval,filename);
 	}
-#else
-	return TRUE;
-#endif
+#else /* HAVE_GNOME_VFS */
+	{
+		struct stat naamstat;
+		errno = 0;
+		retval = ((stat(ondiskencoding, &naamstat) == 0) && (errno == 0));
+		DEBUG_MSG("file_exists_and_readable, retval=%d (ernno=%d) for %s\n",retval,errno,ondiskencoding);
+	}
+#endif /* HAVE_GNOME_VFS */
+	g_free(ondiskencoding);
+#endif /* WIN32 */
+	return retval;
 }
 /**
  * return_first_existing_filename:
