@@ -24,6 +24,7 @@
 /* #define DEBUG */
 
 #include "bluefish.h"
+#include "html2.h" /* style_but_new() */
 #include "html_diag.h" /* myself */
 #include "gtk_easy.h" /* window_full() */
 #include "bf_lib.h"
@@ -37,6 +38,7 @@ Trecent_attribs recent_attribs;
 /*****************************************/
 
 void html_diag_destroy_cb(GtkWidget * widget, Thtml_diag *dg) {
+	dg->tobedestroyed = TRUE;
 	DEBUG_MSG("html_diag_destroy_cb, widget=%p, dg=%p, dg->dialog=%p\n",widget,dg,dg->dialog);
 	if (dg->mark_ins) {
 		gtk_text_buffer_delete_mark(dg->doc->buffer,dg->mark_ins);
@@ -51,10 +53,11 @@ void html_diag_cancel_clicked_cb(GtkWidget *widget, gpointer data) {
 	html_diag_destroy_cb(NULL, data);
 }
 
-Thtml_diag *html_diag_new(gchar *title) {
+Thtml_diag *html_diag_new(Tbfwin *bfwin, gchar *title) {
 	Thtml_diag *dg;
 	
 	dg = g_malloc(sizeof(Thtml_diag));
+	dg->tobedestroyed = FALSE;
 	DEBUG_MSG("html_diag_new, dg=%p\n",dg);
 	dg->dialog = window_full(title, GTK_WIN_POS_MOUSE
 		, 12,G_CALLBACK(html_diag_destroy_cb), dg, TRUE);
@@ -65,7 +68,7 @@ Thtml_diag *html_diag_new(gchar *title) {
 	gtk_container_add(GTK_CONTAINER(dg->dialog), dg->vbox);
 #ifdef DEBUG
 	{
-		Tdocument *doc = main_v->current_document;
+		Tdocument *doc = bfwin->current_document;
 		GtkTextIter iter1, iter2;
 		GtkTextMark* mark;
 		mark = gtk_text_buffer_get_mark(doc->buffer,"insert");
@@ -76,14 +79,14 @@ Thtml_diag *html_diag_new(gchar *title) {
 	}
 #endif
 	{
-		if (!gtk_text_buffer_get_mark(main_v->current_document->buffer,"diag_ins")) {
+		if (!gtk_text_buffer_get_mark(bfwin->current_document->buffer,"diag_ins")) {
 			GtkTextIter iter;
-			GtkTextMark* mark = gtk_text_buffer_get_mark(main_v->current_document->buffer,"insert");
-			gtk_text_buffer_get_iter_at_mark(main_v->current_document->buffer,&iter,mark);
-			dg->mark_ins = gtk_text_buffer_create_mark(main_v->current_document->buffer,"diag_ins",&iter,TRUE);
-			mark = gtk_text_buffer_get_mark(main_v->current_document->buffer,"selection_bound");
-			gtk_text_buffer_get_iter_at_mark(main_v->current_document->buffer,&iter,mark);
-			dg->mark_sel = gtk_text_buffer_create_mark(main_v->current_document->buffer,"diag_sel",&iter,TRUE);
+			GtkTextMark* mark = gtk_text_buffer_get_mark(bfwin->current_document->buffer,"insert");
+			gtk_text_buffer_get_iter_at_mark(bfwin->current_document->buffer,&iter,mark);
+			dg->mark_ins = gtk_text_buffer_create_mark(bfwin->current_document->buffer,"diag_ins",&iter,TRUE);
+			mark = gtk_text_buffer_get_mark(bfwin->current_document->buffer,"selection_bound");
+			gtk_text_buffer_get_iter_at_mark(bfwin->current_document->buffer,&iter,mark);
+			dg->mark_sel = gtk_text_buffer_create_mark(bfwin->current_document->buffer,"diag_sel",&iter,TRUE);
 		} else {
 			dg->mark_ins = dg->mark_sel = NULL;
 		}
@@ -93,11 +96,12 @@ Thtml_diag *html_diag_new(gchar *title) {
 	if (main_v->props.transient_htdialogs) {
 		/* must be set before realizing */
 		DEBUG_MSG("html_diag_finish, setting dg->dialog=%p transient!\n", dg->dialog);
-		gtk_window_set_transient_for(GTK_WINDOW(dg->dialog), GTK_WINDOW(main_v->main_window));
+		gtk_window_set_transient_for(GTK_WINDOW(dg->dialog), GTK_WINDOW(bfwin->main_window));
 	}
 
 	gtk_widget_realize(dg->dialog);
-	dg->doc = main_v->current_document;
+	dg->bfwin = bfwin;
+	dg->doc = bfwin->current_document;
 	return dg;
 }
 
@@ -169,8 +173,7 @@ void parse_html_for_dialogvalues(gchar *dialogitems[], gchar *dialogvalues[]
 }
 
 void fill_dialogvalues(gchar *dialogitems[], gchar *dialogvalues[]
-		, gchar **custom, Ttagpopup *data, GtkWidget *sending_widget
-		, Thtml_diag *diag) {
+		, gchar **custom, Ttagpopup *data, Thtml_diag *diag) {
 
 	gint count=0;
 	
@@ -183,7 +186,7 @@ void fill_dialogvalues(gchar *dialogitems[], gchar *dialogvalues[]
 	 * is called by the right-click-opup, so data will contain info about
 	 * the position of the tag */
 
-	if (data && !sending_widget) {
+	if (data) {
 		parse_html_for_dialogvalues(dialogitems, dialogvalues, custom, data);
 		diag->range.pos = data->pos;
 		diag->range.end = data->end;
@@ -305,16 +308,14 @@ gchar *insert_string_if_entry(GtkWidget * entry, gchar * itemname, gchar * strin
 	return string2add2;
 }
 
-gchar *insert_integer_if_spin(GtkWidget * spin, gchar * itemname, gchar * string2add2, GtkWidget * percentage)
-{
-
-	gchar *tempstring;
-
-	if (strcmp(itemname, cap("BORDER")) == 0 || strcmp(gtk_entry_get_text(GTK_ENTRY(spin)), "0") != 0) {
-		if ((percentage) && (GTK_TOGGLE_BUTTON(percentage)->active)) {
-			tempstring = g_strdup_printf("%s %s=\"%d%%\"", string2add2, itemname, gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin)));
+gchar *insert_integer_if_spin(GtkWidget * spin, gchar * itemname, gchar * string2add2, gboolean ispercentage, gint dontinsertonvalue) {
+	gint val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));
+	if (val != dontinsertonvalue) {
+		gchar *tempstring;
+		if (ispercentage) {
+			tempstring = g_strdup_printf("%s %s=\"%d%%\"", string2add2, itemname, val);
 		} else {
-			tempstring = g_strdup_printf("%s %s=\"%d\"", string2add2, itemname, gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin)));
+			tempstring = g_strdup_printf("%s %s=\"%d\"", string2add2, itemname, val);
 		}
 		g_free(string2add2);
 		string2add2 = tempstring;
@@ -355,3 +356,30 @@ GList *add_entry_to_stringlist(GList *which_list, GtkWidget *entry) {
 	return which_list;
 }
 
+GtkWidget *generic_table_inside_notebookframe(GtkWidget *notebook, const gchar *title, gint rows, gint cols) {
+	GtkWidget *dgtable, *frame = bf_generic_frame_new(NULL, GTK_SHADOW_NONE, 12);
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, gtk_label_new(title));
+	dgtable = gtk_table_new(rows, cols, FALSE);
+	gtk_table_set_row_spacings(GTK_TABLE(dgtable), 6);
+	gtk_table_set_col_spacings(GTK_TABLE(dgtable), 12);
+	gtk_container_add(GTK_CONTAINER(frame), dgtable);
+	return dgtable;
+}
+
+void generic_class_id_style_section(Thtml_diag *dg, gint firstattrwidget, GtkWidget *dgtable, gint firstrowintable, gchar **tagvalues, gint firsttagvalue) {
+	GtkWidget *but;
+
+	dg->attrwidget[firstattrwidget] = combo_with_popdown(tagvalues[firsttagvalue], dg->bfwin->session->classlist, 1);
+	bf_mnemonic_label_tad_with_alignment(_("Cl_ass:"), dg->attrwidget[firstattrwidget], 0, 0.5, dgtable, 0, 1, firstrowintable+0, firstrowintable+1);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[firstattrwidget], 1, 3, firstrowintable+0, firstrowintable+1);
+
+	dg->attrwidget[firstattrwidget+1] = entry_with_text(tagvalues[firsttagvalue+1], 256);
+	bf_mnemonic_label_tad_with_alignment(_("_ID:"), dg->attrwidget[firstattrwidget+1], 0, 0.5, dgtable, 0, 1, firstrowintable+1, firstrowintable+2);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[firstattrwidget+1], 1, 3, firstrowintable+1, firstrowintable+2);
+
+	dg->attrwidget[firstattrwidget+2] = entry_with_text(tagvalues[firsttagvalue+2], 256);
+	bf_mnemonic_label_tad_with_alignment(_("St_yle:"), dg->attrwidget[firstattrwidget+2], 0, 0.5, dgtable, 0, 1, firstrowintable+2, firstrowintable+3);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[firstattrwidget+2], 1, 2, firstrowintable+2, firstrowintable+3);
+	but = style_but_new(dg->attrwidget[firstattrwidget+2], dg->dialog);
+	gtk_table_attach(GTK_TABLE(dgtable), but, 2, 3, firstrowintable+2, firstrowintable+3, GTK_SHRINK, GTK_SHRINK, 0, 0);
+}
