@@ -1,7 +1,7 @@
 #include <gtk/gtk.h>
 #include <string.h>
 
-/* #define DEBUG */
+/*#define DEBUG*/
 
 #include "bluefish.h"
 #include "html_diag.h"
@@ -83,11 +83,11 @@ static void image_insert_dialogok_lcb(GtkWidget * widget, Timage_diag *imdg) {
 		thestring = insert_string_if_entry(imdg->dg->entry[0], cap("SRC"), thestring, NULL);
 	}
 
-	thestring = insert_integer_if_spin(imdg->dg->spin[0], cap("WIDTH"), thestring, imdg->dg->check[0]);
-	thestring = insert_integer_if_spin(imdg->dg->spin[1], cap("HEIGHT"), thestring, imdg->dg->check[1]);
-	thestring = insert_integer_if_spin(imdg->dg->spin[2], cap("BORDER"), thestring, NULL);
-	thestring = insert_integer_if_spin(imdg->dg->spin[3], cap("HSPACE"), thestring, NULL);
-	thestring = insert_integer_if_spin(imdg->dg->spin[4], cap("VSPACE"), thestring, NULL);
+	thestring = insert_integer_if_spin(imdg->dg->spin[0], cap("WIDTH"), thestring, GTK_TOGGLE_BUTTON(imdg->dg->check[0])->active,0);
+	thestring = insert_integer_if_spin(imdg->dg->spin[1], cap("HEIGHT"), thestring, GTK_TOGGLE_BUTTON(imdg->dg->check[1])->active,0);
+	thestring = insert_integer_if_spin(imdg->dg->spin[2], cap("BORDER"), thestring, FALSE, 1);
+	thestring = insert_integer_if_spin(imdg->dg->spin[3], cap("HSPACE"), thestring, FALSE,0);
+	thestring = insert_integer_if_spin(imdg->dg->spin[4], cap("VSPACE"), thestring, FALSE,0);
 	thestring = insert_string_if_entry(imdg->dg->entry[1], cap("NAME"), thestring, NULL);
 	thestring = insert_string_if_entry(imdg->dg->entry[2], cap("ALT"), thestring, "");
 	thestring = insert_string_if_entry(imdg->dg->entry[3], cap("USEMAP"), thestring, NULL);
@@ -212,7 +212,7 @@ static void image_adjust_changed(GtkAdjustment * adj, Timage_diag *imdg) {
 	DEBUG_MSG("image_adjust_changed finished. GTK_IS_WIDGET(imdg->im) == %d\n", GTK_IS_WIDGET(imdg->im));
 }
 
-void image_insert_dialog_backend(gchar *filename,GtkWidget *widget, gpointer data, gboolean is_thumbnail) {
+void image_insert_dialog_backend(gchar *filename,Tbfwin *bfwin, Ttagpopup *data, gboolean is_thumbnail) {
 	gchar *tagvalues[11];
 	gchar *custom = NULL;
 	Timage_diag *imdg;
@@ -224,13 +224,13 @@ void image_insert_dialog_backend(gchar *filename,GtkWidget *widget, gpointer dat
 	imdg->im = NULL;
 	imdg->is_thumbnail = is_thumbnail;
 	if (is_thumbnail) {
-		imdg->dg = html_diag_new(_("Insert thumbnail"));
+		imdg->dg = html_diag_new(bfwin,_("Insert thumbnail"));
 	} else {
-		imdg->dg = html_diag_new(_("Insert image"));
+		imdg->dg = html_diag_new(bfwin,_("Insert image"));
 	}
 	{
 		static gchar *tagitems[] = { "width", "height", "alt", "border", "src", "hspace", "vspace", "align", "name", "usemap", NULL };
-		fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, imdg->dg);
+		fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, imdg->dg);
 	}
 
 	imdg->frame = gtk_frame_new(_("Preview"));
@@ -255,7 +255,7 @@ void image_insert_dialog_backend(gchar *filename,GtkWidget *widget, gpointer dat
 	}
 	bf_mnemonic_label_tad_with_alignment(_("_Image location:"), imdg->dg->entry[0], 0, 0.5, dgtable, 0, 1, 0, 1); 
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->entry[0], 1, 7, 0, 1);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), file_but_new(imdg->dg->entry[0], imdg->dg->dialog, 0), 7, 9, 0, 1);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), file_but_new(imdg->dg->entry[0], 0, bfwin), 7, 9, 0, 1);
 	g_signal_connect(G_OBJECT(imdg->dg->entry[0]), "changed", G_CALLBACK(image_filename_changed), imdg);
 
 	imdg->dg->spin[0] = spinbut_with_value(NULL, 0, 5000, 1.0, 10.0);
@@ -319,14 +319,14 @@ void image_insert_dialog_backend(gchar *filename,GtkWidget *widget, gpointer dat
 	if (custom)	g_free(custom);
 }
 
-void image_insert_dialog_cb(GtkWidget * widget, gpointer data) {
-	image_insert_dialog_backend(NULL, widget, data, FALSE);
+void image_insert_dialog(Tbfwin *bfwin, Ttagpopup *data) {
+	image_insert_dialog_backend(NULL, bfwin, data, FALSE);
 }
-void thumbnail_insert_dialog_cb(GtkWidget * widget, gpointer data) {
-	image_insert_dialog_backend(NULL, NULL, NULL, TRUE);
+void thumbnail_insert_dialog(Tbfwin *bfwin) {
+	image_insert_dialog_backend(NULL, bfwin, NULL, TRUE);
 }
-void image_insert_from_filename(gchar *filename) {
-	image_insert_dialog_backend(filename,NULL, NULL, FALSE);
+void image_insert_from_filename(Tbfwin *bfwin, gchar *filename) {
+	image_insert_dialog_backend(filename,bfwin, NULL, FALSE);
 }
 
 typedef struct {
@@ -336,9 +336,11 @@ typedef struct {
 	GtkWidget *spins[2];
 	GtkTextBuffer *tbuf;
 	gint mode;
+	Tbfwin *bfwin;
 } Tmuthudia;
 
-static void multi_thumbnail_dialog_destroy(GtkWidget *wid, GdkEvent *event, Tmuthudia *mtd) {
+static void multi_thumbnail_dialog_destroy(GtkWidget *wid, Tmuthudia *mtd) {
+	DEBUG_MSG("multi_thumbnail_dialog_destroy, called for mtd=%p\n",mtd);
 	window_destroy(mtd->win);
 	g_free(mtd);
 }
@@ -376,8 +378,8 @@ static void multi_thumbnail_ok_clicked(GtkWidget *widget, Tmuthudia *mtd) {
 		GError *error=NULL;
 		gchar *thumbfilename, *filename=(gchar *)tmplist->data, *relfilename;
 	
-		if (main_v->current_document->filename) {
-			relfilename = create_relative_link_to(main_v->current_document->filename, filename);
+		if (mtd->bfwin->current_document->filename) {
+			relfilename = create_relative_link_to(mtd->bfwin->current_document->filename, filename);
 		} else {
 			relfilename = g_path_get_basename(filename);
 		}
@@ -467,15 +469,15 @@ static void multi_thumbnail_ok_clicked(GtkWidget *widget, Tmuthudia *mtd) {
 		tmplist = g_list_next(tmplist);
 	}
 	DEBUG_MSG("done with all the files, inserting totalstring %s\n", string2insert);
-	doc_insert_two_strings(main_v->current_document, string2insert, NULL);
+	doc_insert_two_strings(mtd->bfwin->current_document, string2insert, NULL);
 	g_free(string2insert);
 	DEBUG_MSG("ready, cleanup time!\n");
 	free_stringlist(files);
-	multi_thumbnail_dialog_destroy(NULL, NULL, mtd);
+	multi_thumbnail_dialog_destroy(NULL, mtd);
 }
 
 static void multi_thumbnail_cancel_clicked(GtkWidget *widget, Tmuthudia *mtd) {
-	multi_thumbnail_dialog_destroy(NULL, NULL, mtd);
+	multi_thumbnail_dialog_destroy(NULL, mtd);
 }
 
 static void multi_thumbnail_radio_toggled_lcb(GtkToggleButton *togglebutton,Tmuthudia *mtd) {
@@ -501,13 +503,15 @@ static void multi_thumbnail_radio_toggled_lcb(GtkToggleButton *togglebutton,Tmut
 	}
 }
 
-void multi_thumbnail_dialog_cb(GtkWidget * widget, gpointer data) {
+void multi_thumbnail_dialog(Tbfwin *bfwin) {
 	Tmuthudia *mtd;
 	GtkWidget *vbox, *hbox, *but, *table, *label, *scrolwin, *textview;
 	gint tb;
 	
 	mtd = g_new(Tmuthudia, 1);
-	mtd->win = window_full(_("Multi thumbnail"), GTK_WIN_POS_MOUSE, 5, G_CALLBACK(multi_thumbnail_dialog_destroy), mtd, TRUE);
+	mtd->bfwin = bfwin;
+	mtd->win = window_full2(_("Multi thumbnail"), GTK_WIN_POS_MOUSE, 5
+		, G_CALLBACK(multi_thumbnail_dialog_destroy), mtd, TRUE, NULL);
 	vbox = gtk_vbox_new(FALSE,5);
 	gtk_container_add(GTK_CONTAINER(mtd->win), vbox);
 	
