@@ -9,26 +9,30 @@ typedef struct {
 	const guint8 *data;
 } Tpixmap;
 
+typedef struct {
+	const guint id;
+	const gchar *stock_id;
+} Tstockpixmap;
+
 static Tpixmap tp[] = {
-	{0, new},
-	{1, open},
-	{2, save},
-	{3, save_as},
-	{4, close},
-	{5, copy},
-	{6, cut},
-	{7, paste},
-	{8, search},
-	{9, search_replace},
-	{10, undo},
-	{11, redo},
-	{12, preferences},
+ 	{0, new}, 
+/* 	{1, open}, */
+/* 	{2, save}, */
+/* 	{3, save_as}, */
+ 	{4, close}, 
+/* 	{5, copy}, */
+/* 	{6, cut}, */
+/* 	{7, paste}, */
+/* 	{8, search}, */
+/*	{9, search_replace}, */
+/* 	{10, undo}, */
+/* 	{11, redo}, */
+/* 	{12, preferences}, */
 	{13, quick_start},
 	{14, body},
 	{15, bold},
 	{16, italic},
 	{17, underline},
-/*	{18, strikeout},*/
 	{19, paragraph},
 	{20, brbreak},
 	{21, breakall},
@@ -37,7 +41,7 @@ static Tpixmap tp[] = {
 	{24, hrule},
 	{25, image},
 	{26, thumbnail},
-/*	{27, multi_thumbnail},  multi thumbnail image ?? */
+	{27, multithumbs}, /*  multi thumbnail image ?? */
 	{28, center},
 	{29, right},
 	{30, comment},
@@ -114,12 +118,16 @@ static Tpixmap tp[] = {
 	{101, close_small},
 	{102, view_in_browser},
 	{103, context}, /* context dropdown image */
-	{104, spellcheck},
+ 	{104, bookmarks},
 	{105, filetree},
 	{106, fref},
+	{107, frinfo},
+	{108, frdialog},
+	{109, frsearch},
 	{110, indent},
 	{111, unindent},
 	{112, open_small},
+	{113, edit_tag},
 	{150,prefseditor},
 	{151,prefsexternal},
 	{152,prefsfiles},
@@ -132,23 +140,40 @@ static Tpixmap tp[] = {
 	{159, nothing}
 };
 
+static Tstockpixmap tsp[] = {
+	{1000, GTK_STOCK_FIND},
+	{1001, GTK_STOCK_FIND_AND_REPLACE}
+};
+
 GtkWidget *new_pixmap(gint type) {
-	GdkPixbuf *pixbuf;
 	GtkWidget *widget;
 	gint i;
-
-	for (i=0;i<(sizeof(tp)/sizeof(Tpixmap));i++) {
-		if (tp[i].id == type) {
-			break;
+	if (type < 1000) {
+		GdkPixbuf *pixbuf;
+		for (i=0;i<(sizeof(tp)/sizeof(Tpixmap));i++) {
+			if (tp[i].id == type) {
+				break;
+			}
 		}
+		if (i >= sizeof(tp)/sizeof(Tpixmap)) {
+			g_print("new_pixmap, requested unknown type %d\n", type);
+			i = sizeof(tp)/sizeof(Tpixmap)-1;
+		}
+		pixbuf = gdk_pixbuf_new_from_inline(-1,tp[i].data,FALSE,NULL);
+		widget = gtk_image_new_from_pixbuf(pixbuf);
+		g_object_unref(pixbuf);
+	} else {
+		for (i=0;i<(sizeof(tsp)/sizeof(Tstockpixmap));i++) {
+			if (tsp[i].id == type) {
+				break;
+			}
+		}
+		if (i >= sizeof(tp)/sizeof(Tpixmap)) {
+			g_print("new_pixmap, requested unknown type %d\n", type);
+			i = sizeof(tp)/sizeof(Tpixmap)-1;
+		}
+		widget = gtk_image_new_from_stock(tsp[i].stock_id,GTK_ICON_SIZE_BUTTON);
 	}
-	if (i >= sizeof(tp)/sizeof(Tpixmap)) {
-		g_print("new_pixmap, requested unknown type %d\n", type);
-		i = sizeof(tp)/sizeof(Tpixmap)-1;
-	}
-	pixbuf = gdk_pixbuf_new_from_inline(-1,tp[i].data,FALSE,NULL);
-	widget = gtk_image_new_from_pixbuf(pixbuf);
-	g_object_unref(pixbuf);
 	return widget;
 }
 
