@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * rcfile.c - loading and parsing of the configfiles
  *
- * Copyright (C) 2000-2002 Olivier Sessink
+ * Copyright (C) 2000-2004 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,9 +40,10 @@
 
 typedef struct {
 	void *pointer; /* where should the value be stored ?*/
-	unsigned char type; /* a=arraylist, l=stringlist, s=string, e=string with escape, i=integer */
+	unsigned char type; /* a=arraylist, l=stringlist, s=string, e=string with escape, i=integer, m=limiTed stringlist */
 	gchar *identifier; /* the string that should be in the config file for this entry */
-	gint len; /* only used for arrays, the length the array should have */
+	gint len; /* used for arrays and limitedstringlists, the length the list should have (only during save)
+						, or the number of items the array should have (during load) */
 } Tconfig_list_item;
 
 static GList *main_configlist=NULL;
@@ -75,37 +76,50 @@ GList *make_config_list_item(GList * config_list, void *pointer_to_var, unsigned
 	return (GList *) g_list_append(config_list, config_list_item);
 }
 
-static void init_prop_integer(GList ** config_list, void *pointer_to_var, gchar * name_of_var, gint default_value)
+static void init_prop_integer(GList ** config_list, void *pointer_to_var, gchar * name_of_var, gint default_value, gboolean set_default)
 {
 	*config_list = make_config_list_item(*config_list, pointer_to_var, 'i', name_of_var, 0);
-	*(gint *) pointer_to_var = default_value;
+	if (set_default) *(gint *)pointer_to_var = default_value;
 }
 
-static void init_prop_string(GList ** config_list, void *pointer_to_var, gchar * name_of_var, gchar * default_value)
+static void init_prop_string(GList ** config_list, void *pointer_to_var, gchar * name_of_var, const gchar * default_value)
 {
 	*config_list = make_config_list_item(*config_list, pointer_to_var, 's', name_of_var, 0);
-	*(gchar **) pointer_to_var = g_strdup(default_value);
-	DEBUG_MSG("init_prop_string, name_of_var=%s, default_value=%s\n", name_of_var, default_value);
+	if (*(gchar **) pointer_to_var == NULL) {
+		*(gchar **) pointer_to_var = g_strdup(default_value);
+	}
+	DEBUG_MSG("init_prop_string, name_of_var=%s, default_value=%s, current value=%s\n", name_of_var, default_value, *(gchar **) pointer_to_var);
 }
 
 static void init_prop_string_with_escape(GList ** config_list, void *pointer_to_var, gchar * name_of_var, gchar * default_value)
 {
 	*config_list = make_config_list_item(*config_list, pointer_to_var, 'e', name_of_var, 0);
-	*(gchar **) pointer_to_var = unescapestring(default_value);
+	*(gchar **) pointer_to_var = unescape_string(default_value, FALSE);
 	DEBUG_MSG("init_prop_string, name_of_var=%s, default_value=%s\n", name_of_var, default_value);
 }
 
-static void init_prop_stringlist(GList ** config_list, void *pointer_to_var, gchar * name_of_var)
+static void init_prop_stringlist(GList ** config_list, void *pointer_to_var, gchar * name_of_var, gboolean setNULL)
 {
 	*config_list = make_config_list_item(*config_list, pointer_to_var, 'l', name_of_var, 0);
-	pointer_to_var = NULL;
+	if (setNULL) {
+	 	pointer_to_var = NULL;
+	}
 }
 
-
-static void init_prop_arraylist(GList ** config_list, void *pointer_to_var, gchar * name_of_var, gint len)
+static void init_prop_arraylist(GList ** config_list, void *pointer_to_var, gchar * name_of_var, gint len, gboolean setNULL)
 {
 	*config_list = make_config_list_item(*config_list, pointer_to_var, 'a', name_of_var, len);
-	pointer_to_var = NULL;
+	if (setNULL) {
+	 	pointer_to_var = NULL;
+	}
+}
+
+static void init_prop_limitedstringlist(GList ** config_list, void *pointer_to_var, gchar * name_of_var, gint len, gboolean setNULL)
+{
+	*config_list = make_config_list_item(*config_list, pointer_to_var, 'm', name_of_var, len);
+	if (setNULL) {
+	 	pointer_to_var = NULL;
+	}
 }
 
 static gint save_config_file(GList * config_list, gchar * filename)
@@ -150,7 +164,7 @@ static gint save_config_file(GList * config_list, gchar * filename)
 			DEBUG_MSG("save_config_file, converting \"%p\"\n", tmpitem);
 			DEBUG_MSG("save_config_file, converting \"%s %s\"\n", tmpitem->identifier, (gchar *) * (void **) tmpitem->pointer);
 			if (*(void **) tmpitem->pointer) {
-				tmpstring2 = escapestring((gchar*)*(void**)tmpitem->pointer, '\0');
+				tmpstring2 = escape_string((gchar*)*(void**)tmpitem->pointer, FALSE);
 				tmpstring = g_strdup_printf("%s %s", tmpitem->identifier, tmpstring2);
 
 				DEBUG_MSG("save_config_file, adding %s\n", tmpstring);
@@ -160,15 +174,21 @@ static gint save_config_file(GList * config_list, gchar * filename)
 			}
 			break;
 		case 'l':
-			DEBUG_MSG("save_config_file, tmpitem(%p), &tmpitem=%p\n", tmpitem, &tmpitem);
-			tmplist2 = g_list_last((GList *) * (void **) tmpitem->pointer);
+		case 'm':
+			DEBUG_MSG("save_config_file, type %c, tmpitem(%p), &tmpitem=%p\n", tmpitem->type, tmpitem, &tmpitem);
+			tmplist2 = NULL;
+			if (tmpitem->type == 'm') {
+				tmplist2 = g_list_nth((GList *) * (void **) tmpitem->pointer,tmpitem->len);
+			}
+			if (!tmplist2) {
+				tmplist2 = g_list_last((GList *) * (void **) tmpitem->pointer);
+			}
 			DEBUG_MSG("save_config_file, the tmplist2(%p)\n", tmplist2);
 			while (tmplist2 != NULL) {
 				tmpstring2 = (char *) tmplist2->data;
 				DEBUG_MSG("save_config_file, tmpstring2(%p)=%s\n", tmpstring2, tmpstring2);
 				tmpstring = g_strdup_printf("%s %s", tmpitem->identifier, tmpstring2);
 				DEBUG_MSG("save_config_file, tmpstring(%p)=%s\n", tmpstring, tmpstring);
-
 				rclist = g_list_append(rclist, tmpstring);
 				tmplist2 = g_list_previous(tmplist2);
 			}
@@ -178,7 +198,7 @@ static gint save_config_file(GList * config_list, gchar * filename)
 			tmplist2 = g_list_last((GList *) * (void **) tmpitem->pointer);
 			DEBUG_MSG("save_config_file, the tmplist2(%p)\n", tmplist2);
 			while (tmplist2 != NULL) {
-				tmpstring2 = array_to_string((char **) tmplist2->data, ':');
+				tmpstring2 = array_to_string((char **) tmplist2->data);
 				tmpstring = g_strdup_printf("%s %s", tmpitem->identifier, tmpstring2);
 				DEBUG_MSG("save_config_file, tmpstring(%p)=%s\n", tmpstring, tmpstring);
 				rclist = g_list_append(rclist, tmpstring);
@@ -200,6 +220,7 @@ static gint save_config_file(GList * config_list, gchar * filename)
 
 static gboolean parse_config_file(GList * config_list, gchar * filename)
 {
+	gboolean retval = FALSE;
 	gchar *tmpstring = NULL, *tmpstring2;
 	gchar **tmparray;
 	GList *rclist, *tmplist, *tmplist2;
@@ -211,7 +232,8 @@ static gboolean parse_config_file(GList * config_list, gchar * filename)
 	rclist = get_list(filename, rclist,FALSE);
 	
 	if (rclist == NULL) {
-		return FALSE;
+		DEBUG_MSG("no rclist, returning!\n");
+		return retval;
 	}
 
 	/* empty all variables that have type GList ('l') */
@@ -248,6 +270,7 @@ static gboolean parse_config_file(GList * config_list, gchar * filename)
 #endif
 				if (g_strncasecmp(tmpitem->identifier, tmpstring, strlen(tmpitem->identifier)) == 0) {
 					/* we have found the correct identifier */
+					retval = TRUE;
 					DEBUG_MSG("parse_config_file, identifier=%s, string=%s\n", tmpitem->identifier, tmpstring);
 					/* move pointer past the identifier */
 					tmpstring += strlen(tmpitem->identifier);
@@ -263,18 +286,19 @@ static gboolean parse_config_file(GList * config_list, gchar * filename)
 						strcpy((char *) *(void **) tmpitem->pointer, tmpstring);
 						break;
 					case 'e':
-						tmpstring2 = unescapestring(tmpstring);
+						tmpstring2 = unescape_string(tmpstring, FALSE); /* I wonder if that should be TRUE */
 						*(void **) tmpitem->pointer = (char *) realloc((char *) *(void **) tmpitem->pointer, strlen(tmpstring2) + 1);
 						strcpy((char *) *(void **) tmpitem->pointer, tmpstring2);
 						g_free(tmpstring2);
 						break;
 					case 'l':
+					case 'm':
 						tmpstring2 = g_strdup(tmpstring);
 						* (void **) tmpitem->pointer = g_list_prepend((GList *) * (void **) tmpitem->pointer, tmpstring2);
 						DEBUG_MSG("parse_config_file, *(void **)tmpitem->pointer=%p\n", *(void **) tmpitem->pointer);
 						break;
 					case 'a':
-						tmparray = string_to_array(tmpstring, ':');
+						tmparray = string_to_array(tmpstring);
 						if (tmpitem->len <= 0 || tmpitem->len == count_array(tmparray)) {
 							* (void **) tmpitem->pointer = g_list_prepend((GList *) * (void **) tmpitem->pointer, tmparray);
 						} else {
@@ -295,113 +319,96 @@ static gboolean parse_config_file(GList * config_list, gchar * filename)
 	}
 	DEBUG_MSG("parse_config_file, parsed all entries, freeing list read from file\n");	
 	free_stringlist(rclist);
-	return TRUE;
+	return retval;
 }
 
 static GList *props_init_main(GList * config_rc)
 {
 /* these are used in the gtk-2 port already */
-	init_prop_integer   (&config_rc, &main_v->props.view_html_toolbar, "view_html_toolbar:", 1);
-	init_prop_integer   (&config_rc, &main_v->props.view_custom_menu, "view_custom_menu:", 1);
-	init_prop_integer   (&config_rc, &main_v->props.view_main_toolbar, "view_main_toolbar:", 1);
-	init_prop_integer   (&config_rc, &main_v->props.view_left_panel, "view_left_panel:", 1);
-	init_prop_integer   (&config_rc, &main_v->props.view_line_numbers, "view_line_numbers:", 1);
-	init_prop_integer   (&config_rc, &main_v->props.filebrowser_show_hidden_files, "fb_show_hidden_f:", 1);
-	init_prop_integer   (&config_rc, &main_v->props.filebrowser_show_others_files, "fb_show_others_f:", 1);
-	init_prop_integer   (&config_rc, &main_v->props.filebrowser_show_backup_files, "fb_show_backup_f:", 0);
+	init_prop_integer   (&config_rc, &main_v->props.view_html_toolbar, "view_html_toolbar:", 1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.view_custom_menu, "view_custom_menu:", 1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.view_main_toolbar, "view_main_toolbar:", 1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.view_left_panel, "view_left_panel:", 1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.view_line_numbers, "view_line_numbers:", 1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.filebrowser_show_hidden_files, "fb_show_hidden_f:", 1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.filebrowser_show_backup_files, "fb_show_backup_f:", 0, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.filebrowser_two_pane_view, "fb_two_pane_view:", 0, TRUE);
 	init_prop_string    (&config_rc, &main_v->props.filebrowser_unknown_icon, "fb_unknown_icon:", PKGDATADIR"icon_unknown.png");
 	init_prop_string    (&config_rc, &main_v->props.filebrowser_dir_icon, "fb_dir_icon:", PKGDATADIR"icon_dir.png");
 	init_prop_string    (&config_rc, &main_v->props.editor_font_string, "editor_font_string:", "courier 11");
-	init_prop_integer   (&config_rc, &main_v->props.editor_tab_width, "editor_tab_width:", 3);
+	init_prop_integer   (&config_rc, &main_v->props.editor_tab_width, "editor_tab_width:", 3, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.editor_indent_wspaces, "editor_indent_wspaces:", 0, TRUE);
 	init_prop_string    (&config_rc, &main_v->props.tab_font_string, "tab_font_string:", "");
-	init_prop_arraylist (&config_rc, &main_v->props.browsers, "browsers:", 2);
-	init_prop_arraylist (&config_rc, &main_v->props.external_commands, "external_commands:", 2);
-	init_prop_stringlist(&config_rc, &main_v->props.quickbar_items, "quickbar_items:");
-	init_prop_integer   (&config_rc, &main_v->props.highlight_num_lines_count, "highlight_num_lines_count:", 1);
-	init_prop_integer   (&config_rc, &main_v->props.defaulthighlight, "defaulthighlight:", 1);
+	init_prop_arraylist (&config_rc, &main_v->props.browsers, "browsers:", 2, TRUE);
+	init_prop_arraylist (&config_rc, &main_v->props.external_commands, "external_commands:", 2, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.highlight_num_lines_count, "highlight_num_lines_count:", 5, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.defaulthighlight, "defaulthighlight:", 1, TRUE);
 	/* old type filetypes have a different count, they are converted below */
-	init_prop_arraylist (&config_rc, &main_v->props.filetypes, "filetypes:", 0);
-	init_prop_integer   (&config_rc, &main_v->props.numcharsforfiletype, "numcharsforfiletype:", 200);
-	init_prop_arraylist (&config_rc, &main_v->props.filefilters, "filefilters:", 3);
+	init_prop_arraylist (&config_rc, &main_v->props.filetypes, "filetypes:", 0, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.numcharsforfiletype, "numcharsforfiletype:", 200, TRUE);
+	init_prop_arraylist (&config_rc, &main_v->props.filefilters, "filefilters:", 3, TRUE);
 	init_prop_string    (&config_rc, &main_v->props.last_filefilter, "last_filefilter:", "");
-	init_prop_integer   (&config_rc, &main_v->props.transient_htdialogs, "transient_htdialogs:", 1);
-	init_prop_integer   (&config_rc, &main_v->props.restore_dimensions, "restore_dimensions:", 1);	
-	init_prop_integer   (&config_rc, &main_v->props.left_panel_width, "left_panel_width:", 150);	
-	init_prop_integer   (&config_rc, &main_v->props.main_window_h, "main_window_height:", 400);
-	init_prop_integer   (&config_rc, &main_v->props.main_window_w, "main_window_width:", 600);
-	init_prop_integer   (&config_rc, &main_v->props.max_recent_files, "max_recent_files:", 15);
-	init_prop_integer   (&config_rc, &main_v->props.max_dir_history, "max_dir_history:", 10);
-	init_prop_integer   (&config_rc, &main_v->props.backup_file,"backup_file:",1);
+	init_prop_integer   (&config_rc, &main_v->props.transient_htdialogs, "transient_htdialogs:", 1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.restore_dimensions, "restore_dimensions:", 1, TRUE);	
+	init_prop_integer   (&config_rc, &main_v->props.left_panel_width, "left_panel_width:", 150, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.max_recent_files, "max_recent_files:", 15, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.max_dir_history, "max_dir_history:", 10, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.backup_file,"backup_file:",1, TRUE);
 	init_prop_string    (&config_rc, &main_v->props.backup_filestring,"backup_filestring:","~");
-	init_prop_integer    (&config_rc, &main_v->props.backup_abort_action,"backup_abort_action:",DOCUMENT_BACKUP_ABORT_ASK);
-	init_prop_integer   (&config_rc, &main_v->props.backup_cleanuponclose,"backup_cleanuponclose:",0);
+	init_prop_integer   (&config_rc, &main_v->props.backup_abort_action,"backup_abort_action:",DOCUMENT_BACKUP_ABORT_ASK, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.backup_cleanuponclose,"backup_cleanuponclose:",0, TRUE);
 	init_prop_string    (&config_rc, &main_v->props.image_thumbnailstring, "image_thumbnailstring:", "_thumbnail");
 	init_prop_string    (&config_rc, &main_v->props.image_thumbnailtype, "image_thumbnailtype:", "png");
-	init_prop_integer   (&config_rc, &main_v->props.image_thumbnail_refresh_quality,"image_thumbnail_refresh_quality:",1);
-	init_prop_integer   (&config_rc, &main_v->props.image_thumbnailsizing_type,"image_thumbnailsizing_type:",0);
-	init_prop_integer   (&config_rc, &main_v->props.image_thumbnailsizing_val1,"image_thumbnailsizing_val1:",100);
-	init_prop_integer   (&config_rc, &main_v->props.image_thumbnailsizing_val2,"image_thumbnailsizing_val2:",100);
+	init_prop_integer   (&config_rc, &main_v->props.image_thumbnail_refresh_quality,"image_thumbnail_refresh_quality:",1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.image_thumbnailsizing_type,"image_thumbnailsizing_type:",0, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.image_thumbnailsizing_val1,"image_thumbnailsizing_val1:",100, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.image_thumbnailsizing_val2,"image_thumbnailsizing_val2:",100, TRUE);
 	init_prop_string    (&config_rc, &main_v->props.image_thumnailformatstring,"image_thumnailformatstring:","<a href=\"%r\"><img src=\"%t\" width=\"%x\" height=\"%y\" border=\"0\"></a>");
-	init_prop_integer   (&config_rc, &main_v->props.allow_multi_instances,"allow_multi_instances:",0);
-	init_prop_integer   (&config_rc, &main_v->props.num_undo_levels,"num_undo_levels:",100);
-	init_prop_integer   (&config_rc, &main_v->props.clear_undo_on_save,"clear_undo_on_save:",0);
+	init_prop_integer   (&config_rc, &main_v->props.allow_multi_instances,"allow_multi_instances:",0, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.modified_check_type,"modified_check_type:",1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.num_undo_levels,"num_undo_levels:",100, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.clear_undo_on_save,"clear_undo_on_save:",0, TRUE);
 	init_prop_string    (&config_rc, &main_v->props.newfile_default_encoding,"newfile_default_encoding:","UTF-8");
-	init_prop_arraylist (&config_rc, &main_v->props.encodings, "encodings:", 2);
-	init_prop_integer   (&config_rc, &main_v->props.auto_set_encoding_meta,"auto_set_encoding_meta:",1);
-	init_prop_integer   (&config_rc, &main_v->props.auto_update_meta,"auto_update_meta:",1);
-	init_prop_arraylist (&config_rc, &main_v->props.outputbox, "outputbox:", 7);
-	init_prop_integer   (&config_rc, &main_v->props.ext_browsers_in_submenu,"ext_browsers_in_submenu:",0);
-	init_prop_integer   (&config_rc, &main_v->props.ext_commands_in_submenu,"ext_commands_in_submenu:",1);
-	init_prop_integer   (&config_rc, &main_v->props.ext_outputbox_in_submenu,"ext_outputbox_in_submenu:",1);
-	init_prop_arraylist (&config_rc, &main_v->props.reference_files, "reference_files:", 2);
-	init_prop_integer   (&config_rc, &main_v->props.fref_ldoubleclick_action,"fref_ldoubleclick_action:",0);
-	init_prop_integer   (&config_rc, &main_v->props.fref_info_type,"fref_info_type:",0);	
-	init_prop_integer   (&config_rc, &main_v->props.document_tabposition,"document_tabposition:",(gint)GTK_POS_BOTTOM);
-	init_prop_integer   (&config_rc, &main_v->props.leftpanel_tabposition,"leftpanel_tabposition:",(gint)GTK_POS_BOTTOM);
+	init_prop_arraylist (&config_rc, &main_v->props.encodings, "encodings:", 2, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.auto_set_encoding_meta,"auto_set_encoding_meta:",1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.auto_update_meta,"auto_update_meta:",1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.encoding_search_Nbytes, "encoding_search_Nbytes:", 500, TRUE);
+	init_prop_arraylist (&config_rc, &main_v->props.outputbox, "outputbox:", 7, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.ext_browsers_in_submenu,"ext_browsers_in_submenu:",0, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.ext_commands_in_submenu,"ext_commands_in_submenu:",1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.ext_outputbox_in_submenu,"ext_outputbox_in_submenu:",1, TRUE);
+	init_prop_arraylist (&config_rc, &main_v->props.reference_files, "reference_files:", 2, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.bookmarks_sort,"bookmarks_sort:",1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.bookmarks_default_store,"bookmarks_default_store:",1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.bookmarks_filename_mode,"bookmarks_filename_mode:",1, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.document_tabposition,"document_tabposition:",(gint)GTK_POS_BOTTOM, TRUE);
+	init_prop_integer   (&config_rc, &main_v->props.leftpanel_tabposition,"leftpanel_tabposition:",(gint)GTK_POS_BOTTOM, TRUE);
+	init_prop_string    (&config_rc, &main_v->props.default_basedir,"default_basedir:",g_get_home_dir());
+	init_prop_string    (&config_rc, &main_v->props.project_suffix,"project_suffix:",".bfproject");
 #ifdef HAVE_LIBASPELL
 	init_prop_string(&config_rc, &main_v->props.spell_default_lang, "spell_default_lang:", "en");
 #endif /* HAVE_LIBASPELL */
-	init_prop_integer   (&config_rc, &main_v->props.default_advanced_snr,"default_advanced_snr:",0);
-	init_prop_integer(&config_rc, &main_v->props.cont_highlight_full, "cont_highlight_full:", 1);
-	init_prop_integer(&config_rc, &main_v->props.lasttime_cust_menu, "lasttime_cust_menu:", 0);
-	init_prop_integer(&config_rc, &main_v->props.lasttime_highlighting, "lasttime_highlighting:", 0);
-	init_prop_integer(&config_rc, &main_v->props.lasttime_filetypes, "lasttime_filetypes:", 0);
-	init_prop_integer(&config_rc, &main_v->props.lasttime_encodings, "lasttime_encodings:", 0);
-
 	/* not yet in use */
 	init_prop_string(&config_rc, &main_v->props.image_editor_cline, "image_editor_command:", "gimp-remote -n \"%s\"&");
-	init_prop_integer(&config_rc, &main_v->props.full_p, "closing_paragraph_tag:", 1);
-	init_prop_integer(&config_rc, &main_v->props.full_li, "closing_list_item_tag:", 1);
-	init_prop_integer(&config_rc, &main_v->props.allow_css, "allow_the_use_of_css:", 1);
-	init_prop_integer(&config_rc, &main_v->props.allow_dep, "allow_the_use_of_font:", 1);
-	init_prop_integer(&config_rc, &main_v->props.format_by_context, "format_by_context:", 1);
-	init_prop_integer(&config_rc, &main_v->props.xhtml, "use_xhtml:", 0);
-	init_prop_integer(&config_rc, &main_v->props.allow_ruby, "allow_the_use_of_ruby:", 0);
-	init_prop_integer(&config_rc, &main_v->props.allow_h4, "allow_the_use_of_html4:", 1);
-	init_prop_integer(&config_rc, &main_v->props.allow_frames, "allow_the_use_of_frames:", 1);
-	init_prop_integer(&config_rc, &main_v->props.force_dtd, "force_an_dtd:", 0);
-	init_prop_integer(&config_rc, &main_v->props.dtd_url, "url_in_dtd:", 0);
-	init_prop_integer(&config_rc, &main_v->props.xml_start, "xml_starting_line:", 0);
-	init_prop_integer(&config_rc, &main_v->props.lowercase_tags, "lowercase_tags:", 1);
-	init_prop_integer(&config_rc, &main_v->props.word_wrap, "word_wrap:", 0);
-	init_prop_integer(&config_rc, &main_v->props.autoindent, "autoindent:", 1);
-	init_prop_integer(&config_rc, &main_v->props.fontset, "fontset:", 0);
-	init_prop_integer(&config_rc, &main_v->props.force_def_style, "force_white_background:", 1);
-	init_prop_integer(&config_rc, &main_v->props.drop_at_drop_pos, "drop_at_drop_position:", 0);
-	init_prop_integer(&config_rc, &main_v->props.link_management, "link_management:", 1);
+	init_prop_integer(&config_rc, &main_v->props.allow_dep, "allow_the_use_of_font:", 0, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.format_by_context, "format_by_context:", 1, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.xhtml, "use_xhtml:", 0, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.allow_ruby, "allow_the_use_of_ruby:", 0, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.force_dtd, "force_an_dtd:", 0, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.dtd_url, "url_in_dtd:", 0, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.xml_start, "xml_starting_line:", 0, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.lowercase_tags, "lowercase_tags:", 1, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.word_wrap, "word_wrap:", 0, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.autoindent, "autoindent:", 1, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.drop_at_drop_pos, "drop_at_drop_position:", 0, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.link_management, "link_management:", 1, TRUE);
 	
-	init_prop_integer(&config_rc, &main_v->props.cont_highlight_update, "continuous_highlight_update:", 1);
-	init_prop_string(&config_rc, &main_v->props.html_ver, "used_html_version:", "HTML 4.0 Transitional");
-	/* key conversion */
-	init_prop_integer (&config_rc, &main_v->props.conv_ctrl_enter,"conv_ctrl_enter:",0);
-	init_prop_integer (&config_rc, &main_v->props.conv_shift_enter,"conv_shift_enter:",0);
-	init_prop_string_with_escape (&config_rc, &main_v->props.shift_enter_text,"shift_enter_text:","<br>\\n");
-	init_prop_string_with_escape (&config_rc, &main_v->props.ctrl_enter_text,"ctrl_enter_text:","\\n</p>\\n<p>");
-	init_prop_integer (&config_rc, &main_v->props.conv_special_char,"conv_special_char:",0);
 #ifdef WITH_MSG_QUEUE
-	init_prop_integer (&config_rc, &main_v->props.open_in_running_bluefish,"open_in_running_bluefish:",1);
+	init_prop_integer (&config_rc, &main_v->props.open_in_running_bluefish,"open_in_running_bluefish:",1, TRUE);
+#endif
+#ifdef HAVE_GNOME_VFS
+	init_prop_integer (&config_rc, &main_v->props.server_zope_compat,"server_zope_compat:", 0, TRUE); 
 #endif
-
 	return config_rc;
 }
 
@@ -510,9 +517,9 @@ void rcfile_parse_main(void)
 				g_print("Unable to find '"PKGDATADIR"encodings.default'\n");
 			}
 		} else {
-			if (config_file_is_newer(main_v->props.lasttime_encodings,defaultfile)) {
+			if (config_file_is_newer(main_v->globses.lasttime_encodings,defaultfile)) {
 				main_v->props.encodings = arraylist_load_new_identifiers_from_file(main_v->props.encodings,defaultfile,1);
-				main_v->props.lasttime_encodings = TIME_T_TO_GINT(time(NULL));
+				main_v->globses.lasttime_encodings = TIME_T_TO_GINT(time(NULL));
 			}
 		}
 		g_free(defaultfile);
@@ -534,8 +541,8 @@ void rcfile_parse_main(void)
 		main_v->props.external_commands = g_list_append(main_v->props.external_commands,arr);
 	}
 	{
-	gchar *defaultfile = return_first_existing_filename(PKGDATADIR"filetypes.default",
-										"data/filetypes.default",
+		gchar *defaultfile = return_first_existing_filename(PKGDATADIR"filetypes.default",
+									"data/filetypes.default",
 									"../data/filetypes.default",NULL);
 		if (main_v->props.filetypes == NULL) {
 			/* if the user does not have file-types --> set them to defaults values */
@@ -545,9 +552,9 @@ void rcfile_parse_main(void)
 				g_print("Unable to find '"PKGDATADIR"filetypes.default'\n");
 			}
 		} else {
-			if (config_file_is_newer(main_v->props.lasttime_filetypes,defaultfile)) {
+			if (config_file_is_newer(main_v->globses.lasttime_filetypes,defaultfile)) {
 				main_v->props.filetypes = arraylist_load_new_identifiers_from_file(main_v->props.filetypes,defaultfile,1);
-				main_v->props.lasttime_filetypes = TIME_T_TO_GINT(time(NULL));
+				main_v->globses.lasttime_filetypes = TIME_T_TO_GINT(time(NULL));
 			}
 		}
 		g_free(defaultfile);
@@ -573,16 +580,28 @@ void rcfile_parse_main(void)
 		fref_rescan_dir(userdir);
 		g_free(userdir);
 	}
-	/* for backwards compatibility with old filetypes */
+	/* for backwards compatibility with old filetypes, 
+		before version 0.10 had length 4, 
+		before version 0.13 had length 6 */
 	{
 		GList *tmplist = g_list_first(main_v->props.filetypes);
 		while (tmplist) {
 			gchar **orig = (gchar **)tmplist->data;
 			if (count_array(orig)==4) {
-				gchar **new = array_from_arglist(orig[0], orig[1], orig[2], orig[3], "1", "", NULL);
+				gchar **new = array_from_arglist(orig[0], orig[1], orig[2], orig[3], "1", "", "1", NULL);
 				tmplist->data = new;
 				g_strfreev(orig);
 			}
+			if (count_array(orig)==6) {
+				gchar **new = array_from_arglist(orig[0], orig[1], orig[2], orig[3], orig[4], orig[5], "0", NULL);
+				tmplist->data = new;
+				if (strcmp(orig[0], "xml")==0) {
+					new[6][0] = '1';
+				} else if (strcmp(orig[0], "html")==0 || strcmp(orig[0], "php")==0 || strcmp(orig[0], "jsp")==0 || strcmp(orig[0], "cfml")==0) {
+					new[6][0] = '2';
+				}
+				g_strfreev(orig);
+			}
 			tmplist = g_list_next(tmplist);
 		}
 	}
@@ -611,7 +630,7 @@ void rcfile_parse_highlighting(void) {
 	DEBUG_MSG("rcfile_parse_highlighting, started\n");
 
 	highlighting_configlist = NULL;
-	init_prop_arraylist(&highlighting_configlist, &main_v->props.highlight_patterns, "patterns:", 0);
+	init_prop_arraylist(&highlighting_configlist, &main_v->props.highlight_patterns, "patterns:", 0, TRUE);
 
 	filename = g_strconcat(g_get_home_dir(), "/.bluefish/highlighting", NULL);
 	defaultfile = return_first_existing_filename(PKGDATADIR"highlighting.default",
@@ -627,9 +646,9 @@ void rcfile_parse_highlighting(void) {
 		save_config_file(highlighting_configlist, filename);
 		DEBUG_MSG("rcfile_parse_highlighting, done saving\n");
 	} else {
-		if (config_file_is_newer(main_v->props.lasttime_highlighting,defaultfile)) {
+		if (config_file_is_newer(main_v->globses.lasttime_highlighting,defaultfile)) {
 			main_v->props.highlight_patterns = arraylist_load_new_identifiers_from_file(main_v->props.highlight_patterns,defaultfile,2);
-			main_v->props.lasttime_highlighting = TIME_T_TO_GINT(time(NULL));
+			main_v->globses.lasttime_highlighting = TIME_T_TO_GINT(time(NULL));
 		}
 	}
 	g_free(filename);
@@ -647,19 +666,19 @@ static gint rcfile_save_highlighting(void) {
 void rcfile_parse_custom_menu(void) {
 	gchar *filename;
 	gchar *defaultfile;
-	GList *cust_menu = NULL;
 	DEBUG_MSG("rcfile_parse_custom_menu, started\n");
 
 	custom_menu_configlist = NULL;
-	init_prop_arraylist(&custom_menu_configlist, &cust_menu, "custom_menu:", 0);
-	init_prop_arraylist(&custom_menu_configlist, &main_v->props.cmenu_insert, "cmenu_insert:", 0);
-	init_prop_arraylist(&custom_menu_configlist, &main_v->props.cmenu_replace, "cmenu_replace:", 0);
+	init_prop_arraylist(&custom_menu_configlist, &main_v->props.cust_menu, "custom_menu:", 0, TRUE);
+	init_prop_arraylist(&custom_menu_configlist, &main_v->props.cmenu_insert, "cmenu_insert:", 0, TRUE);
+	init_prop_arraylist(&custom_menu_configlist, &main_v->props.cmenu_replace, "cmenu_replace:", 0, TRUE);
 
 	filename = g_strconcat(g_get_home_dir(), "/.bluefish/custom_menu", NULL);
 	defaultfile = return_first_existing_filename(PKGDATADIR"custom_menu.default",
 									"data/custom_menu.default",
 									"../data/custom_menu.default",NULL);
 	if (!parse_config_file(custom_menu_configlist, filename) || (main_v->props.cust_menu==NULL && main_v->props.cmenu_insert==NULL && main_v->props.cmenu_replace==NULL )) {
+		DEBUG_MSG("error parsing the custom menu file\n");
 		/* init the custom_menu in some way? */
 		if (defaultfile) {
 			parse_config_file(custom_menu_configlist, defaultfile);
@@ -667,14 +686,15 @@ void rcfile_parse_custom_menu(void) {
 			g_print("Unable to find '"PKGDATADIR"custom_menu.default'\n");
 		}
 	} else {
-		if (config_file_is_newer(main_v->props.lasttime_cust_menu,defaultfile)) {
+		if (config_file_is_newer(main_v->globses.lasttime_cust_menu,defaultfile)) {
 			GList *default_insert=NULL, *default_replace=NULL, *tmp_configlist=NULL;
-			init_prop_arraylist(&tmp_configlist, &default_insert, "cmenu_insert:", 0);
-			init_prop_arraylist(&tmp_configlist, &default_replace, "cmenu_replace:", 0);
+			DEBUG_MSG("config_file_is_newer!\n");
+			init_prop_arraylist(&tmp_configlist, &default_insert, "cmenu_insert:", 0, TRUE);
+			init_prop_arraylist(&tmp_configlist, &default_replace, "cmenu_replace:", 0, TRUE);
 			parse_config_file(tmp_configlist, defaultfile);
 			main_v->props.cmenu_insert = arraylist_load_new_identifiers_from_list(main_v->props.cmenu_insert, default_insert, 1);
 			main_v->props.cmenu_replace = arraylist_load_new_identifiers_from_list(main_v->props.cmenu_replace, default_replace, 1);
-			main_v->props.lasttime_cust_menu = TIME_T_TO_GINT(time(NULL));
+			main_v->globses.lasttime_cust_menu = TIME_T_TO_GINT(time(NULL));
 			free_arraylist(default_replace);
 			free_arraylist(default_insert);
 			free_configlist(tmp_configlist);
@@ -683,11 +703,13 @@ void rcfile_parse_custom_menu(void) {
 	/* for backwards compatibility with older (before Bluefish 0.10) custom menu files we can convert those.. 
 	we will not need the 'type' anymore, since we will put them in separate lists, hence the memmove() call
 	*/
-	if (cust_menu) {
-		GList *tmplist= g_list_first(cust_menu);
+	DEBUG_MSG("main_v->props.cust_menu=%p\n",main_v->props.cust_menu);
+	if (main_v->props.cust_menu) {
+		GList *tmplist= g_list_first(main_v->props.cust_menu);
 		while (tmplist) {
 			gchar **strarr = (gchar **)tmplist->data;
 			gint count = count_array(strarr);
+			DEBUG_MSG("converting cust_menu, found count=%d\n",count);
 			if (count >= 5 && strarr[1][0] == '0') {
 				DEBUG_MSG("rcfile_parse_custom_menu, converting insert, 0=%s, 1=%s\n", strarr[0], strarr[1]);
 				g_free(strarr[1]);
@@ -718,7 +740,8 @@ void rcfile_parse_custom_menu(void) {
 			}
 			tmplist = g_list_next(tmplist);
 		}
-		g_list_free(cust_menu);
+		g_list_free(main_v->props.cust_menu);
+		main_v->props.cust_menu=NULL;
 	}
 	
 /*		main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, array_from_arglist(N_("/php/ibase/ibase fetch row"), "0", "while ($%1 = ibase_fetch_row($%0)) {\n	echo $%1[0];\n", "\n}\nibase_free_result($%0);\n", "2", _("ibase result variable name"), _("row variable name"), NULL));
@@ -811,12 +834,112 @@ void rcfile_save_configfile_menu_cb(gpointer callback_data,guint action,GtkWidge
 #endif
 	break;
 	}
-
 }
 
 void rcfile_save_all(void) {
 	rcfile_save_main();
 	rcfile_save_highlighting();
 	rcfile_save_custom_menu();
+	rcfile_save_global_session();
 }
 
+static GList *return_globalsession_configlist(gboolean init_values) {
+	GList *config_rc = NULL;
+	init_prop_stringlist(&config_rc, &main_v->globses.quickbar_items, "quickbar_items:", TRUE);
+	init_prop_integer   (&config_rc, &main_v->globses.two_pane_filebrowser_height, "two_pane_filebrowser_height:", 250, init_values);
+	init_prop_integer   (&config_rc, &main_v->globses.main_window_h, "main_window_height:", 400, init_values);
+	init_prop_integer   (&config_rc, &main_v->globses.main_window_w, "main_window_width:", 600, init_values); /* negative width means maximized */
+	init_prop_integer   (&config_rc, &main_v->globses.fref_ldoubleclick_action,"fref_ldoubleclick_action:",0, init_values);
+	init_prop_integer   (&config_rc, &main_v->globses.fref_info_type,"fref_info_type:",0, init_values);
+	init_prop_integer   (&config_rc, &main_v->globses.lasttime_cust_menu, "lasttime_cust_menu:", 0, init_values);
+	init_prop_integer   (&config_rc, &main_v->globses.lasttime_highlighting, "lasttime_highlighting:", 0, init_values);
+	init_prop_integer   (&config_rc, &main_v->globses.lasttime_filetypes, "lasttime_filetypes:", 0, init_values);
+	init_prop_integer   (&config_rc, &main_v->globses.lasttime_encodings, "lasttime_encodings:", 0, init_values);
+	init_prop_limitedstringlist(&config_rc, &main_v->globses.recent_projects, "recent_projects:", main_v->props.max_recent_files, FALSE);
+	return config_rc;
+}
+
+static GList *return_session_configlist(GList *configlist, Tsessionvars *session) {
+	init_prop_stringlist(&configlist, &session->searchlist, "searchlist:", FALSE);
+	init_prop_stringlist(&configlist, &session->replacelist, "replacelist:", FALSE);
+	init_prop_stringlist(&configlist, &session->classlist, "classlist:", FALSE);
+	init_prop_stringlist(&configlist, &session->colorlist, "colorlist:", FALSE);
+	init_prop_stringlist(&configlist, &session->targetlist, "targetlist:", FALSE);
+	init_prop_stringlist(&configlist, &session->urllist, "urllist:", FALSE);
+	init_prop_stringlist(&configlist, &session->fontlist, "fontlist:", FALSE);
+	init_prop_stringlist(&configlist, &session->dtd_cblist, "dtd_cblist:", FALSE);
+	init_prop_arraylist (&configlist, &session->bmarks, "bmarks:", 6, FALSE); /* what is the lenght for a bookmark array? */
+	init_prop_limitedstringlist(&configlist, &session->recent_files, "recent_files:", main_v->props.max_recent_files, FALSE);
+	init_prop_limitedstringlist(&configlist, &session->recent_dirs, "recent_dirs:", main_v->props.max_dir_history, FALSE);
+	return configlist;
+}
+
+static GList *return_project_configlist(Tproject *project) {
+	GList *configlist = NULL;
+	init_prop_string(&configlist, &project->name,"name:",_("Untitled Project"));
+	init_prop_stringlist(&configlist, &project->files, "files:", FALSE);
+	init_prop_string(&configlist, &project->basedir,"basedir:","");
+	init_prop_string(&configlist, &project->webdir,"webdir:","");
+	init_prop_string(&configlist, &project->template,"template:","");
+/*	init_prop_stringlist(&configlist, &project->recentfiles, "recentfiles:", FALSE); / * should be changed to use the session->recent_files */
+	init_prop_integer (&configlist, &project->view_main_toolbar,"view_main_toolbar:",1,FALSE);
+	init_prop_integer (&configlist, &project->view_left_panel,"view_left_panel:",1,FALSE);
+	init_prop_integer (&configlist, &project->view_custom_menu,"view_custom_menu:",1,FALSE);
+	init_prop_integer (&configlist, &project->view_html_toolbar,"view_html_toolbar:",1,FALSE);
+	init_prop_integer (&configlist, &project->word_wrap,"word_wrap:",1,FALSE);
+	configlist = return_session_configlist(configlist, project->session);
+	return configlist;
+}
+
+gboolean rcfile_parse_project(Tproject *project, gchar *filename) {
+	gboolean retval;
+	GList *configlist = return_project_configlist(project);
+	retval = parse_config_file(configlist, filename);
+	free_configlist(configlist);
+	return retval;
+}
+
+gboolean rcfile_save_project(Tproject *project, gchar *filename) {
+	gboolean retval;
+	GList *configlist = return_project_configlist(project);
+	DEBUG_MSG("rcfile_save_project, project %p, name='%s', basedir='%s', webdir='%s'\n",project, project->name, project->basedir, project->webdir);
+	DEBUG_MSG("rcfile_save_project, bmarks=%p, list length=%d\n",project->session->bmarks, g_list_length(project->session->bmarks));
+	DEBUG_MSG("rcfile_save_project, length session recent_files=%d\n",g_list_length(project->session->recent_files));
+	retval = save_config_file(configlist, filename);
+	free_configlist(configlist);
+	return retval;
+}
+
+gboolean rcfile_save_global_session(void) {
+	gboolean retval;
+	gchar *filename = g_strconcat(g_get_home_dir(), "/.bluefish/session", NULL);
+	GList *configlist = return_globalsession_configlist(FALSE);
+	configlist = return_session_configlist(configlist, main_v->session);
+	DEBUG_MSG("rcfile_save_global_session, saving global session to %s\n",filename);
+	DEBUG_MSG("rcfile_save_global_session, length session recent_files=%d\n",g_list_length(main_v->session->recent_files));
+	DEBUG_MSG("rcfile_save_global_session, length session recent_projects=%d\n",g_list_length(main_v->globses.recent_projects));
+	DEBUG_MSG("rcfile_save_global_session, main window width=%d\n",main_v->globses.main_window_w);
+	retval = save_config_file(configlist, filename);
+	free_configlist(configlist);
+	g_free(filename);
+	return TRUE;
+}
+/* should be called AFTER the normal properties are loaded, becauses return_session_configlist() uses
+ settings from main_v->props */
+gboolean rcfile_parse_global_session(void) {
+	gboolean retval;
+	gchar *filename;
+	GList *configlist = return_globalsession_configlist(TRUE);
+	configlist = return_session_configlist(configlist, main_v->session);
+	filename = g_strconcat(g_get_home_dir(), "/.bluefish/session", NULL);
+	if (!file_exists_and_readable(filename)) {
+		/* versions before 0.13 did not have a separate session file, so 
+		we'll try to load these items from rcfile_v2 */
+		g_free(filename);
+		filename = g_strconcat(g_get_home_dir(), "/.bluefish/rcfile_v2", NULL);
+	}
+	retval = parse_config_file(configlist, filename);
+	free_configlist(configlist);
+	g_free(filename);
+	return retval;
+}
